/**********************************************************************
*
* FileName:        main.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC33Fxxxx
* Compiler:        MPLAB C30 v3.00 or higher
* Tested On:	   dsPIC33FJ12GP201
*
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* Jim Veatch	2/24/11	Initial Release
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* ADDITIONAL NOTES: 
* DSP Code for the ARRL HBCIII Trancsiever
**********************************************************************/

#include "p33FJ12GP201.h"
#include "libq.h" /* include fixed point library */
#include "stdlib.h"
#include "FIR_Filter.h"
#include "SSB_2400.h"
#include "SSB_2200.h"
#include "SSB_2000.h"
#include "SSB_1800.h"
#include "CW_1000.h"
#include "CW_800.h"
#include "CW_600.h"
#include "CW_400.h"
#include "AUDIO_SSB.h"
#include "AUDIO_CW.h"
#define NUM_SAMPLES 1

// Setup the processor configuration Bits:
// Start on the internal FRC oscillator, switch to the Ext Clock/PLL later
_FOSCSEL(FNOSC_FRC);		

// Clock Switching is enabled and Fail Safe Clock Monitor is disabled
// OSC2 Pin Function: OSC2 is Clock Output
// Primary Oscillator Mode: XT Crystal
// Multiple peripheral pin switching enabled
_FOSC(FCKSM_CSECMD & OSCIOFNC_OFF & POSCMD_EC & IOL1WAY_OFF & OSCIOFNC_ON);	

// Watchdog Timer Enabled/disabled by user software
_FWDT(FWDTEN_OFF); 			            			

int  	aoutBuff[1], PD_out[1], PD_out_F[1], phaseAcc[1], outGain;
int  	inSamples[NUM_SAMPLES], outSamples[NUM_SAMPLES];
int		aFlag, LUind, RXchar, SERst, spiFlag, spiData[1], currLevel;
int		CWind, CWout[1];
long	PD_acc;

// Initial Settings
int		TXsel = 0;			// Receive Mode
int		TX_bc = 0;			// TX buffer counter
int		AUDIOsel = '2';		// 3 kHz Audio LPF
int		CW_acc[1] = {0};	// CW phase accumulator
int		ST_gain = 5;		// Sidetone Gain	
int 	PHASEinc = 20972; 	// 16.000 kHz BFO
int		FILTERsel = '7';	// 2.4 kHz IF Filter
int		Level = 0;			// RX level
int		Mode = 'S';			// Set operating mode to SSB
int		old_RB7;			// Previous state of the Key line
int		ZERO[1] = {0};		// Buffer Clearer
long	AGC_F = 0xFFFFFF;	// Initially set to max gain
long	IF_GN = 0xFFFFFF;	// IF gain set to MAX
long	H_Time = 0;			// Hang time counter for the audio detetor
long	HT_load = 0x186A0;	// Hang tine load value (long)

int		sineLU[128] =  {0x0000, 0x0064, 0x00C8, 0x012C, 0x018F, 0x01F1, 0x0251, 0x02B1,
						0x030E, 0x036A, 0x03C4, 0x041B, 0x0470, 0x04C2, 0x0511, 0x055D,
						0x05A5, 0x05EB, 0x062C, 0x066A, 0x06A4, 0x06D9, 0x070B, 0x0738,
						0x0760, 0x0785, 0x07A4, 0x07BF, 0x07D5, 0x07E6, 0x07F2, 0x07FA,
						0x07FC, 0x07FA, 0x07F2, 0x07E6, 0x07D5, 0x07BF, 0x07A4, 0x0785,
						0x0760, 0x0738, 0x070B, 0x06D9, 0x06A4, 0x066A, 0x062C, 0x05EB,
						0x05A5, 0x055D, 0x0511, 0x04C2, 0x0470, 0x041B, 0x03C4, 0x036A,
						0x030E, 0x02B1, 0x0251, 0x01F1, 0x018F, 0x012C, 0x00C8, 0x0064,
						0x0000, 0xFF9C, 0xFF38, 0xFED4, 0xFE71, 0xFE0F, 0xFDAF, 0xFD4F,
						0xFCF2, 0xFC96, 0xFC3C, 0xFBE5, 0xFB90, 0xFB3E, 0xFAEF, 0xFAA3,
						0xFA5B, 0xFA15, 0xF9D4, 0xF996, 0xF95C, 0xF927, 0xF8F5, 0xF8C8,
						0xF8A0, 0xF87B, 0xF85C, 0xF841, 0xF82B, 0xF81A, 0xF80E, 0xF806,
						0xF804, 0xF806, 0xF80E, 0xF81A, 0xF82B, 0xF841, 0xF85C, 0xF87B,
						0xF8A0, 0xF8C8, 0xF8F5, 0xF927, 0xF95C, 0xF996, 0xF9D4, 0xFA15,
						0xFA5B, 0xFAA3, 0xFAEF, 0xFB3E, 0xFB90, 0xFBE5, 0xFC3C, 0xFC96,
						0xFCF2, 0xFD4F, 0xFDAF, 0xFE0F, 0xFE71, 0xFED4, 0xFF38, 0xFF9C};

void Init_ADC(void)
{

		AD1CON1bits.FORM   = 1;		// Data Output Format: Signed Integer
		AD1CON1bits.SSRC   = 2;		// Sample Clock Source: GP Timer 3 starts conversion
		AD1CON1bits.ASAM   = 1;		// ADC Sample Control: Sampling begins immediately after conversion
		AD1CON1bits.AD12B  = 1;		// 12-bit ADC operation

		AD1CON2bits.CSCNA = 0;		// Do not scan inputs
		AD1CON2bits.CHPS  = 0;		// Converts CH1

		AD1CON3bits.ADRC = 0;		// ADC Clock is derived from Systems Clock
		AD1CON3bits.ADCS = 38;		// ADC Conversion Clock Tad=Tcy*(ADCS+1)= (1/40M)*39 = 0.975us (1026Khz)
									// ADC Conversion Time for 12-bit Tc=14*Tab = 13.7us

		AD1CHS0 = 0x01;				// AN1 Selected for conversion
        
        IFS0bits.AD1IF = 0;			// Clear the A/D interrupt flag bit
        IEC0bits.AD1IE = 1;			// Enable A/D interrupt 
        AD1CON1bits.ADON = 1;		// Turn on the A/D converter	
		aFlag = 0;
}

/*=============================================================================  
Timer 3 is setup to time-out every 20 microseconds (50Khz Rate). As a result, the module 
will stop sampling and trigger a conversion on every Timer3 time-out, i.e., Ts=20us. 
=============================================================================*/
void Init_Tmr3(void) 
{
		// Initialize Output Compare Module
		OC1CONbits.OCM = 0b000; // Disable Output Compare Module
		OC1CONbits.OCTSEL = 0; 	// Select Timer 2 as output compare time base
		OC1R = 5; 				// Load the Compare Register Value for rising edge
		OC1RS = 790; 			// Load the Compare Register Value for falling edge
		IFS0bits.OC1IF = 0; 	// Clear Output Compare 1 Interrupt Flag
		OC1CONbits.OCM = 0b101; // Select the Output Compare mode        TMR2 = 0x0000;
        PR2 = 799;
        IFS0bits.T2IF = 0;
        IEC0bits.T2IE = 0;
        T2CONbits.TON = 1;
		int temp;	
		for (temp=0;temp<190;temp++)
        TMR3 = 0x0000;
        PR3 = 799;
        IFS0bits.T3IF = 0;
        IEC0bits.T3IE = 0;
        T3CONbits.TON = 1;		//Start Timer 3
}
void Init_UART(void) 
{
		U1MODE = 0x0000;			// UART Mode Register
		U1STA = 0X0000;				// UART Status Register
		U1BRG = 129;				// 40Mhz osc, 19200 Baud
		IFS0bits.U1TXIF = 0;		// Clear the Transmit Interrupt Flag
		IEC0bits.U1TXIE = 0;		// Disable Transmit Interrupts
		IFS0bits.U1RXIF = 0;		// Clear the Recieve Interrupt Flag
		IEC0bits.U1RXIE = 0;		// Disable Recieve Interrupts
		U1MODEbits.UARTEN = 1;		// And turn the peripheral on
		SERst = 0;					// Serial input status
}

void Init_SPI(void)
{
	// setup the SPI peripheral
	__builtin_write_OSCCONL(0x80);		// Unlock Peripheral Pin Switching
	RPOR4 = 0x0708;						// Clock 0n RP8 (p11), Data on RP9 (p12)
	RPINR18 = 0X1F0F;					// UART RX on RP15 (p16)
	RPOR2 = 0x0012;						// Clock 0n RP8 (p11), Data on RP9 (p12)
	__builtin_write_OSCCONL(0xC0);		// Lock Peripheral Pin Switching

	// setup /CS and /LDAC
	TRISAbits.TRISA4 = 0;				// set PORTA RA4 to output
	PORTAbits.RA4 = 1;					// RA4 (/CS) HIGH
	SPI1STAT = 0x0;  					// disable the SPI module (just in case)
	SPI1CON1 = 0x053A;					// DSICLK = 0, DISSDO = 0, MODE16 = 1; SMP = 0; CKE = 1; SSEN = 0; CKP = 0; MSTEN = 1; SPRE = 0b110, PPRE = 0b10
	SPI1STAT = 0x8000; 					// enable the SPI module
	IFS0bits.SPI1IF = 0; 				// Clear the Interrupt Flag
}

void __attribute__((interrupt, shadow, no_auto_psv)) _SPI1Interrupt(void)
{
	short temp;	
	PORTAbits.RA4 = 1;					// raise the /CS line
	IFS0bits.SPI1IF = 0; 				// Clear the Interrupt Flag
	if (spiFlag == 1)
	{
		spiFlag = 0;					// set flag to SPI first byte
		temp = SPI1BUF;					// dummy read of the SPI1BUF register to clear the SPIRBF flag
		PORTAbits.RA4 = 0;				// lower the /CS line
		SPI1BUF = spiData[0] | 0x7000;	// write the data out to the SPI peripheral
	}
	IEC0bits.SPI1IE = 1;
}
void __attribute__((interrupt, shadow, no_auto_psv)) _ADC1Interrupt(void)
{
    IFS0bits.AD1IF = 0;		// Clear the ADC1 Interrupt Flag
	inSamples[0] = ADC1BUF0;
	short temp;	
	IFS0bits.SPI1IF = 0; 				// Clear the Interrupt Flag
	PORTAbits.RA4 = 0;					// lower the /CS line
	temp = SPI1BUF;						// dummy read of the SPI1BUF register to clear the SPIRBF flag
	SPI1BUF = aoutBuff[0] | 0xf000;		// write the data out to the SPI peripheral
	IEC0bits.SPI1IE = 1;
	spiFlag = 1;						// set flag to SPI second byte
	aFlag = 1;
}
int CalcPhaseInc(int ConvChar)
{
	int PSI = 20972; 
	if (ConvChar >= 65 && ConvChar <= 90) PSI = 20972 + (52 * (ConvChar - 64))/10;
	if (ConvChar >= 97 && ConvChar <=122) PSI = 20972 + (52 * (98 - ConvChar))/10;	
	return(PSI);
}
int main(void)
{

// Configure Oscillator to operate the device at 40Mhz
// Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
// Fosc= 25M*32/(2*5)=80Mhz for 25M input clock
	PLLFBD=30;					// M=32
	CLKDIVbits.PLLPOST=0;		// N1=2
	CLKDIVbits.PLLPRE=3;		// N2=5
	OSCTUN=0;					// Tune FRC oscillator, if FRC is used
	RCONbits.SWDTEN=0;			// Disable Watch Dog Timer

// Clock switching to incorporate PLL
	__builtin_write_OSCCONH(0x03);		// Initiate Clock Switch to Primary
										// Oscillator with PLL (NOSC=0b011)
	__builtin_write_OSCCONL(0x01);		// Start clock switching
	while (OSCCONbits.COSC != 0b011);	// Wait for Clock switch to occur	
	while (OSCCONbits.LOCK!=1) {}; 		// Wait for PLL to lock

	AD1PCFGL = 0b1111111100;	// set the AN0, AN1 to analog, all others digital 
	TRISBbits.TRISB7 = 1;		// set PORTB RB7 to input (TX line) pin 10
	TRISBbits.TRISB14 = 0;		// set PORTB RB17 to output (Audio detect line) pin 15
	TRISAbits.TRISA3 = 1;		// set PORTA RA3 to input (RX Mute line) pin 7

	// Init the SPI port and pins:
	Init_SPI();
	Init_ADC();
	Init_Tmr3();
	Init_UART();

	while(1)
	{
		if(aFlag==1)
		{
			if (TXsel == 0)	// Receiver running...
			{
				// IF Filter
				aFlag = 0;
				if (FILTERsel == '0') FIR( NUM_SAMPLES, &outSamples[0], &inSamples[0],  &CW_400Filter);
			 	if (FILTERsel == '1') FIR( NUM_SAMPLES, &outSamples[0], &inSamples[0],  &CW_600Filter);
			 	if (FILTERsel == '2') FIR( NUM_SAMPLES, &outSamples[0], &inSamples[0],  &CW_800Filter);
				if (FILTERsel == '3') FIR( NUM_SAMPLES, &outSamples[0], &inSamples[0],  &CW_1000Filter);
				if (FILTERsel == '4') FIR( NUM_SAMPLES, &outSamples[0], &inSamples[0],  &SSB_1800Filter);
				if (FILTERsel == '5') FIR( NUM_SAMPLES, &outSamples[0], &inSamples[0],  &SSB_2000Filter);
				if (FILTERsel == '6') FIR( NUM_SAMPLES, &outSamples[0], &inSamples[0],  &SSB_2200Filter);
				if (FILTERsel == '7') FIR( NUM_SAMPLES, &outSamples[0], &inSamples[0],  &SSB_2400Filter);		
			// RX Level Detector
				currLevel = abs(outSamples[0]);
				if (Level <= currLevel) Level = currLevel;
				else Level = Level - 2;
				if (Level < 0) Level = 0;
				if (Level > 0x1FF) Level = 0x01FF;
			// AGC
				if (Level > 175) AGC_F = AGC_F - 5000;
				else 
				{
					if (H_Time >0) AGC_F = AGC_F + 2;
					else AGC_F = AGC_F + 500;
				}
				if (AGC_F > IF_GN) AGC_F = IF_GN;
				if (AGC_F < 0x0) AGC_F = 0x0;
				spiData[0] = AGC_F / 0x1000;
			// LO Generator
				phaseAcc[0]= phaseAcc[0] + PHASEinc;
				LUind = (phaseAcc[0] >> 9) & 0x7F;
			// Product Detector		
				PD_acc =  __builtin_mulss(outSamples[0],sineLU[LUind]);
				PD_out[0] = PD_acc >> 11;
			// Audio Filter
				if (AUDIOsel=='1') FIR( NUM_SAMPLES, &PD_out_F[0], &PD_out[0], &AUDIO_CWFilter);
				else FIR( NUM_SAMPLES, &PD_out_F[0], &PD_out[0], &AUDIO_SSBFilter);
				outGain = PD_out_F[0] * 7;
				if (outGain > 0x0780) outGain = 0x0780; 	// Software Clipper +
				if (outGain < -0x0780) outGain = -0x0780;	// Software Clipper 1
				if (!PORTAbits.RA3) outGain = 0;			// Check audio mute pin
				aoutBuff[0] = outGain + 0x800;
			}
			// Signal present detector
			if (Level >= 165 && PORTAbits.RA3 && TXsel == 0) // Level exceeds threashold
			{
				H_Time = HT_load;		// load the hang time counter
				PORTBbits.RB14 = 1;		// assert pin
			}
			if (Level <= 160 && H_Time > 0 && PORTAbits.RA3 && TXsel == 0) H_Time-- ; //Level below threshold
			if (H_Time == 0) PORTBbits.RB14 = 0; // Times out desert the pin
			if (TXsel == 1 || !PORTAbits.RA3) H_Time = 0; // Keyed or muted? dum the HT counter

			if (TXsel == 1 && Mode == 'S')	// SSB Transmitter running...
			{
				// Audio Filter
				aFlag = 0;
				FIR( NUM_SAMPLES, &outSamples[0], &inSamples[0],  &AUDIO_SSBFilter);		
				// LO Generator
				phaseAcc[0]= phaseAcc[0] + 20972;	// Dump the IF shift offset in TX
				LUind = (phaseAcc[0] >> 9) & 0x7F;
				// Product Detector		
				PD_acc =  __builtin_mulss(outSamples[0]<<2,sineLU[LUind]);
				PD_out[0] = PD_acc >> 11;
				// IF Filter
				FIR( NUM_SAMPLES, &PD_out_F[0], &PD_out[0], &SSB_2200Filter);
				aoutBuff[0] = PD_out_F[0] + 0x07FF;
			}
			if (TXsel == 1 && Mode == 'C')	// CW Transmitter running...
			{
				aFlag = 0;
				// CW Sidetone Generator
				CW_acc[0] = CW_acc[0] + 1048; // 800 Hz sidetone
				CWind = (CW_acc[0] >> 9) & 0x7F;
				CWout[0] = sineLU[CWind] * ST_gain;
				CWout[0] = CWout[0] / 16;
				// CW Generator
				phaseAcc[0] = phaseAcc[0] + 22020;	//16.8 kHz
				LUind = (phaseAcc[0] >> 9) & 0x7F;
				// CW Gain		
				PD_acc =  __builtin_mulss(0x0300,sineLU[LUind]);
				PD_out[0] = PD_acc >> 11;
				aoutBuff[0] = PD_out[0] + CWout[0] + 0x07FF;
			}
			if (TXsel == 2) //Clear Buffers
			{
				FIR( NUM_SAMPLES, &outSamples[0], &ZERO[0],  &AUDIO_SSBFilter);		
				FIR( NUM_SAMPLES, &PD_out_F[0], &ZERO[0], &CW_800Filter);
				aoutBuff[0] = PD_out_F[0] + 0x07FF;
				if (TX_bc++ == 128)
				{
					TX_bc = 0;
					if (PORTBbits.RB7 == 1) 
					{
						TXsel = 0;
						AD1CHS0 = 0x01;	// AN1 Selected for conversion
					}
					else
					{
						TXsel = 1;
						AD1CHS0 = 0x00;	// AN0 Selected for conversion
					}
				}
			}
		}
		if(U1STAbits.URXDA)
		{
		RXchar = U1RXREG;		// if a character is availabel, get it
			if(RXchar == '*')
				SERst=1;
			else
			{
				switch(SERst)
				{
				case 1: // We got a '*' what's the next character
					SERst=0;
					if (RXchar == 'B') SERst=2; //Set IF Bandwidth
					if (RXchar == 'S') SERst=3; //Set LO Offset (IF shift)
					if (RXchar == 'A') SERst=4; //Set Audio Bandwidth
					if (RXchar == 'M') SERst=5; //Set Operating Mode
					if (RXchar == 'T') SERst=6; //Set Sidetone Level
					if (RXchar == 'H') SERst=7; //Set Audio Det Hang Time
					if (RXchar == 'I') SERst=8; //Set IF Gain
					break;
				case 2: // We got a 'B' so set the IF bandwidth
					SERst=0;
					FILTERsel = RXchar;
					break;
				case 3: // We got a 'S' so offset the BFO
					SERst=0;
					PHASEinc = CalcPhaseInc(RXchar);
					break;
				case 4: // We got a 'A' so set the audio filter
					SERst=0;
					AUDIOsel = RXchar;
					break;
				case 5: //We got a 'M' so set the operating mode
					SERst=0;
					Mode = RXchar;
					break;
				case 6: //We got a 'T' so set the sidetone gain
					SERst=0;
					if (RXchar >= '1' && RXchar <= '9') ST_gain = RXchar - 48;
					break;
				case 7: //We got a 'H' so set the audio detector hangtime
					SERst=0;
					if (RXchar == 'S') HT_load = 0x61A8; 	// 0.5 sec
					if (RXchar == 'M') HT_load = 0xC350; 	// 1.0 sec
					if (RXchar == 'L') HT_load = 0x186A0;	// 2.0 sec
					break;
				case 8: //We got a 'I' so set the IF Gain
					SERst=0;
					if (RXchar >= 0x2B && RXchar <= 0xAA) IF_GN = (RXchar - 0x2b) * 0x20000 + 0x1FFFF;
					break;
				default:
					SERst=0;
				}
			}	
		}
		if (PORTBbits.RB7 != old_RB7) // Detect a change in the key line
			{
			old_RB7 = PORTBbits.RB7;
			TXsel = 2;
			}
	}
}

