/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        adcDrv1.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC33Fxxxx
* Compiler:        MPLAB C30 v3.00 or higher
* Tested On:	   dsPIC33FJ12GP201
*
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* Jim Veatch		2/24/11		Initial Release
*
* ADDITIONAL NOTES:
* A/D Routines
**********************************************************************/

#include "p33fxxxx.h"
#include <libq.h> /* include fixed point library */

/*=============================================================================
ADC INITIALIZATION FOR CHANNEL SCAN
=============================================================================*/
void Init_ADC(void)
{

		AD1CON1bits.FORM   = 0;		// Data Output Format: Integer
		AD1CON1bits.SSRC   = 2;		// Sample Clock Source: GP Timer starts conversion
		AD1CON1bits.ASAM   = 1;		// ADC Sample Control: Sampling begins immediately after conversion
		AD1CON1bits.AD12B  = 1;		// 12-bit ADC operation

		AD1CON2bits.CSCNA = 0;		// Do not scan inputs
		AD1CON2bits.CHPS  = 0;		// Converts CH1

		AD1CON3bits.ADRC = 0;		// ADC Clock is derived from Systems Clock
		AD1CON3bits.ADCS = 38;		// ADC Conversion Clock Tad=Tcy*(ADCS+1)= (1/40M)*39 = 0.975us (1026Khz)
									// ADC Conversion Time for 12-bit Tc=14*Tab = 13.7us

		AD1CHS0 = 0x01;				// AN0 Selected for conversion
		
        
        IFS0bits.AD1IF = 0;			// Clear the A/D interrupt flag bit
        IEC0bits.AD1IE = 1;			// Enable A/D interrupt 
        AD1CON1bits.ADON = 1;		// Turn on the A/D converter	

}

/*=============================================================================  
Timer 3 is setup to time-out every 20 microseconds (50Khz Rate). As a result, the module 
will stop sampling and trigger a conversion on every Timer3 time-out, i.e., Ts=20us. 
=============================================================================*/
void Init_Tmr3(void) 
{
        TMR3 = 0x0000;
        PR3 = 799;
        IFS0bits.T3IF = 0;
        IEC0bits.T3IE = 0;

        //Start Timer 3
        T3CONbits.TON = 1;
 
		short temp;	
		for (temp=0;temp<10;temp++);

        TMR2 = 0x0000;
        PR2 = 799;
        IFS0bits.T2IF = 0;
        IEC0bits.T2IE = 0;

        T2CONbits.TON = 1;
}

/*=============================================================================  
ADC INTERRUPT SERVICE ROUTINE
=============================================================================*/

int  ainBuff ;
void __attribute__((interrupt, no_auto_psv)) _ADC1Interrupt(void)
{

    IFS0bits.AD1IF = 0;		// Clear the ADC1 Interrupt Flag

	ainBuff=ADC1BUF0;

	short temp;	

	IFS0bits.SPI1IF = 0; 				// Clear the Interrupt Flag
	PORTAbits.RA4 = 0;					// lower the /CS line
	temp = SPI1BUF;						// dummy read of the SPI1BUF register to clear the SPIRBF flag
	SPI1BUF = ADC1BUF0 + 0xf000;			// write the data out to the SPI peripheral
	IEC0bits.SPI1IE = 1;

}








