/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        main.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC33Fxxxx/PIC24Hxxxx
* Compiler:        MPLAB C30 v3.00 or higher
* Tested On:	   dsPIC33FJ256GP710
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Incorporated ("Microchip") retains all ownership and 
* intellectual property rights in the code accompanying this message and in all 
* derivatives hereto.  You may use this code, and any derivatives created by 
* any person or entity by or on your behalf, exclusively with Microchip's
* proprietary products.  Your acceptance and/or use of this code constitutes 
* agreement to the terms and conditions of this notice.
*
* CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS".  NO 
* WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
* TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIP'S 
* PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
*
* YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE, WHETHER 
* IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), 
* STRICT LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, SPECIAL, 
* PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, FOR COST OR EXPENSE OF 
* ANY KIND WHATSOEVER RELATED TO THE CODE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN 
* ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT 
* ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO 
* THIS CODE, SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO 
* HAVE THIS CODE DEVELOPED.
*
* You agree that you are solely responsible for testing the code and 
* determining its suitability.  Microchip has no obligation to modify, test, 
* certify, or support the code.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* David B. Martin	11/23/06	Initial Release
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* ADDITIONAL NOTES: 
* The Processor starts with the External Crystal without PLL enabled and then the Clock is switched to PLL Mode.
**********************************************************************/

#include "p33Fxxxx.h"


_FOSCSEL(FNOSC_FRC);		
_FOSC(FCKSM_CSECMD & OSCIOFNC_OFF & POSCMD_EC & IOL1WAY_OFF);	
													// Clock Switching is enabled and Fail Safe Clock Monitor is disabled
													// OSC2 Pin Function: OSC2 is Clock Output
													// Primary Oscillator Mode: XT Crystal
													// Multiple peripheral pin switching enabled
_FWDT(FWDTEN_OFF); 			            			// Watchdog Timer Enabled/disabled by user software


void write_SPI(unsigned short command)
{	
	short temp;	

	PORTAbits.RA4 = 0;					// lower the /CS line
	temp = SPI1BUF;						// dummy read of the SPI1BUF register to clear the SPIRBF flag
	SPI1BUF = command + 0xf000;			// write the data out to the SPI peripheral
    while (!IFS0bits.SPI1IF);			// wait for the data to be sent out
	PORTAbits.RA4 = 1;					// raise the /CS line
	IFS0bits.SPI1IF = 0; 				// Clear the Interrupt Flag
	PORTBbits.RB4 = 0;					// RB4 (/LDAC) LOW
	temp = 0;							// Slight Pause
	PORTBbits.RB4 = 1;					// RB4 (/LDAC) HIGH

}

void delay(void)
{	
	short temp;	
	for (temp=0;temp<2;temp++);
		
}	

void Init_SPI(void)
{

	// setup the SPI peripheral
	__builtin_write_OSCCONL(0x80);		// Unlock Peripheral Pin Switching
	RPOR4 = 0x0708;						// Clock 0n RP8 (p11), Data on RP9 (p12)
	__builtin_write_OSCCONL(0xC0);		// Lock Peripheral Pin Switching

	// setup /CS and /LDAC
	TRISAbits.TRISA4 = 0;				// set PORTA RA4 to output
	TRISBbits.TRISB4 = 0;				// set PORTA RB4 to output
	PORTAbits.RA4 = 1;					// RA4 (/CS) HIGH
	PORTBbits.RB4 = 1;					// RB4 (/LDAC) HIGH


	SPI1STAT = 0x0;  					// disable the SPI module (just in case)
	SPI1CON1 = 0x053A;					// DSICLK = 0, DISSDO = 0, MODE16 = 1; SMP = 0; CKE = 1; SSEN = 0; CKP = 0; MSTEN = 1; SPRE = 0b110, PPRE = 0b10
	SPI1STAT = 0x8000; 					// enable the SPI module
	IFS0bits.SPI1IF = 0; 				// Clear the Interrupt Flag
}
int main(void)
{

	short i,j;
	
// Configure Oscillator to operate the device at 40Mhz
// Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
// Fosc= 25M*32/(2*5)=80Mhz for 25M input clock
	PLLFBD=30;					// M=32
	CLKDIVbits.PLLPOST=0;		// N1=2
	CLKDIVbits.PLLPRE=3;		// N2=5
	OSCTUN=0;					// Tune FRC oscillator, if FRC is used

	
// Disable Watch Dog Timer
	RCONbits.SWDTEN=0;

// Clock switching to incorporate PLL
	__builtin_write_OSCCONH(0x03);		// Initiate Clock Switch to Primary
										// Oscillator with PLL (NOSC=0b011)
	__builtin_write_OSCCONL(0x01);		// Start clock switching
	while (OSCCONbits.COSC != 0b011);	// Wait for Clock switch to occur	

// Wait for PLL to lock

	// setup the SPI1 port
	// setup the I/O pins properly (SDI1 as input, SDO1 as output, RG3 as output)
	AD1PCFGLbits.PCFG0 = 1;		// set the RA0 pin to Digital Mode (all others to analog) 
	TRISAbits.TRISA0 = 0;		// set PORTA RA0 to output
	

	while(OSCCONbits.LOCK!=1) {}; /* Wait for PLL to lock*/

	// Init the SPI port and pins:
	Init_SPI();

	while(1)
	{
		//demonstrate the SPI peripheral in Master Mode CKE = 0, CKP = 0
	
	
		for (i=0;i<0x0f;i++)
			{
				write_SPI(i*256);
//				delay();
			} 
	}
}
/******************************************************************************/

