unsigned char ReadEEPROM(unsigned char address);
void WriteEEPROM(unsigned char address,unsigned char data);
void EEPROM_Restore(void);
void EEPROM_Freq (unsigned char bnd, unsigned char vfo);
unsigned char getLSB(int data);
unsigned char getMSB(int data);
void writeIntEE(unsigned char address, unsigned char dataLSB, unsigned char dataMSB);


unsigned char ReadEEPROM(unsigned char address)
	{
    EECON1=0;                   //ensure CFGS=0 and EEPGD=0 
    EEADR = address;
    EECON1bits.RD = 1;
    return(EEDATA);
	}
 
void WriteEEPROM(unsigned char address,unsigned char data)
	{
	char SaveInt;
    SaveInt=INTCON;             //save interrupt status
    EECON1=0;                   //ensure CFGS=0 and EEPGD=0
    EECON1bits.WREN = 1;        //enable write to EEPROM
    EEADR = address;            //setup Address
    EEDATA = data;              //and data
    INTCONbits.GIE=0;           //No interrupts
    EECON2 = 0x55;              //required sequence #1
    EECON2 = 0xaa;              //#2
    EECON1bits.WR = 1;          //#3 = actual write
    INTCON=SaveInt;             //restore interrupts
	while (EECON1bits.WR);
    EECON1bits.WREN = 0;        //disable write to EEPROM
	}

void EEPROM_Restore(void)
	{
	unsigned char eeTemp;
	eeTemp = ReadEEPROM(0x01);
	Band_Offset[0][0] = eeTemp*256;
	eeTemp = ReadEEPROM(0x02);
	Band_Offset[0][0] += eeTemp;

	eeTemp = ReadEEPROM(0x03);
	Band_Offset[0][1] = eeTemp*256;
	eeTemp = ReadEEPROM(0x04);
	Band_Offset[0][1] += eeTemp;

	eeTemp = ReadEEPROM(0x05);
	Band_Offset[1][0] = eeTemp*256;
	eeTemp = ReadEEPROM(0x06);
	Band_Offset[1][0] += eeTemp;

	eeTemp = ReadEEPROM(0x07);
	Band_Offset[1][1] = eeTemp*256;
	eeTemp = ReadEEPROM(0x08);
	Band_Offset[1][1] += eeTemp;

	eeTemp = ReadEEPROM(0x09);
	Band_Offset[2][0] = eeTemp*256;
	eeTemp = ReadEEPROM(0x0a);
	Band_Offset[2][0] += eeTemp;

	eeTemp = ReadEEPROM(0x0b);
	Band_Offset[2][1] = eeTemp*256;
	eeTemp = ReadEEPROM(0x0c);
	Band_Offset[2][1] += eeTemp;

	eeTemp = ReadEEPROM(0x0e);
	if ((eeTemp == 0)|(eeTemp == 1)) A_B[0] = eeTemp;
	else A_B[0] = 0;

	eeTemp = ReadEEPROM(0x0f);
	if ((eeTemp == 0)|(eeTemp == 1)) A_B[1] = eeTemp;
	else A_B[1] = 0;

	eeTemp = ReadEEPROM(0x10);
	if ((eeTemp == 0)|(eeTemp == 1)) A_B[2] = eeTemp;
	else A_B[2] = 0;

	eeTemp = ReadEEPROM(0x11);
	if ((eeTemp == 0)|(eeTemp == 1)|(eeTemp == 2)) Band = eeTemp;
	else Band = 0;
	if (Band == 2) PORTCbits.RC1=1;
	else PORTCbits.RC1=0;

	eeTemp = ReadEEPROM(0x12);
	if ((eeTemp == 0)|(eeTemp == 1)) LNA[0] = eeTemp;
	else LNA[0] = 1;

	eeTemp = ReadEEPROM(0x13);
	if ((eeTemp == 0)|(eeTemp == 1)) LNA[1] = eeTemp;
	else LNA[1] = 1;

	eeTemp = ReadEEPROM(0x14);
	if ((eeTemp == 0)|(eeTemp == 1)) LNA[2] = eeTemp;
	else LNA[2] = 1;

	eeTemp = ReadEEPROM(0x15);
	if (eeTemp < 3) Mode[0][0] = eeTemp;
	else Mode[0][0] = 0;

	eeTemp = ReadEEPROM(0x16);
	if (eeTemp < 3) Mode[1][0] = eeTemp;
	else Mode[1][0] = 0;

	eeTemp = ReadEEPROM(0x17);
	if (eeTemp < 3) Mode[2][0] = eeTemp;
	else Mode[2][0] = 0;

	eeTemp = ReadEEPROM(0x18);
	if (eeTemp < 3) Mode[0][1] = eeTemp;
	else Mode[0][1] = 0;

	eeTemp = ReadEEPROM(0x19);
	if (eeTemp < 3) Mode[1][1] = eeTemp;
	else Mode[1][1] = 0;

	eeTemp = ReadEEPROM(0x1a);
	if (eeTemp < 3) Mode[2][1] = eeTemp;
	else Mode[2][1] = 0;

	eeTemp = ReadEEPROM(0x1b);
	if (eeTemp < 4) Filter[0][0] = eeTemp;
	else Filter[0][0] = 3;

	eeTemp = ReadEEPROM(0x1c);
	if (eeTemp < 4) Filter[1][0] = eeTemp;
	else Filter[1][0] = 3;

	eeTemp = ReadEEPROM(0x1d);
	if (eeTemp < 4) Filter[2][0] = eeTemp;
	else Filter[2][0] = 3;

	eeTemp = ReadEEPROM(0x1e);
	if (eeTemp < 4) Filter[0][1] = eeTemp;
	else Filter[0][1] = 3;

	eeTemp = ReadEEPROM(0x1f);
	if (eeTemp < 4) Filter[1][1] = eeTemp;
	else Filter[1][1] = 3;

	eeTemp = ReadEEPROM(0x20);
	if (eeTemp < 4) Filter[2][1] = eeTemp;
	else Filter[2][1] = 3;

	eeTemp = ReadEEPROM(0x21);
	if (eeTemp < 10) Parm_Data[3] = eeTemp;
	else Parm_Data[3] = 4;

	eeTemp = ReadEEPROM(0x22);
	if (eeTemp < 3) Parm_Data[2] = eeTemp;
	else Parm_Data[2] = 1;

	}

void EEPROM_Freq (unsigned char bnd, unsigned char vfo)
	{
	unsigned char B_Add;
	B_Add = 1 + 2 * (bnd * 2 + vfo);
	writeIntEE(B_Add, getLSB(Band_Offset[bnd][vfo]), getMSB(Band_Offset[bnd][vfo]));
	}
unsigned char getLSB(int data) 
	{return data & 0xFF;}

unsigned char getMSB(int data) 
	{return data / 256u;}

void writeIntEE(unsigned char address, unsigned char dataLSB, unsigned char dataMSB)
	{
	//Configuration as per manual
	EEADR =(address);
	EEDATA = dataMSB; //MSB
	EECON1bits.EEPGD =0;
	EECON1bits.CFGS =0;
	EECON1bits.WREN =1; 
	INTCONbits.GIE = 0;
	EECON2 = 0x55;
	EECON2 = 0xAA; 
	EECON1bits.WR = 1;
	INTCONbits.GIE = 1;
	EECON1bits.WREN = 0;
	while (EECON1bits.WR);
	//Configuration as per manual
	EEADR =(address+1);
	EEDATA = dataLSB; //LSB
	EECON1bits.EEPGD =0;
	EECON1bits.CFGS =0;
	EECON1bits.WREN =1; 
	INTCONbits.GIE = 0;
	EECON2 = 0x55;
	EECON2 = 0xAA; 
	EECON1bits.WR = 1;
	INTCONbits.GIE = 1;
	EECON1bits.WREN = 0;
	while (EECON1bits.WR);
	}
	
