/**********************************************************************
*
* FileName:        main.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       PIC18F45K22
* Compiler:        MPLAB C318 v3.39 or higher
*
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* Jim Veatch	    10/13/11		Initial Release
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* ADDITIONAL NOTES: 
* GP Code for the DSP-610 Trancsiever
**********************************************************************/


// Variables
unsigned int Freq_Dial, Band_Offset[3][2], Disp_Delay = 0, new_RIT;
unsigned int old1_RIT = 0, old2_RIT = 0, fu_lim, fd_lim, F_PWR, R_PWR, PWR;
unsigned int M_Temp, CW_Delay = 0;
unsigned short long Freq_Store = 0;
unsigned long Freq_Data, Freq_Offset, Math_Temp, LO2;
int	Keypad_c = 0, RIT_Freq = 0, Parm_Data[5] = {0,1,1};
char Freq_Disp[10],Parm_Disp[6], P1[4] = "*P1", P0[4] = "*P0", RFG[4] = "*I0" ;
char PE[4] = "*PE", SWR_E = 0;
unsigned char Old_PB, Int_Flag = 0, TEMP, Key;
unsigned char Band, A_B[3], Mode[3][2], Filter[3][2];
unsigned char Rate = 10, LNA[3], TX = 0, TX_Flag = 0, S_meter;
unsigned char slide = 0, Parm_Sel = 0, Not_A_B, Power = 1;
unsigned int PBT_Freq, ADC_Word, i, j, k;
unsigned char new_RFG, old1_RFG = 0, old2_RFG = 0, TXM = 'F';
unsigned char RX_char, oldRX_char = 0;
union
{
struct 
{
unsigned char B1;
unsigned char B2;
unsigned char B3;
unsigned char B4;
};
unsigned long WD;	
} LO1;


#pragma romdata
unsigned long Base_Freq[3] = {2449000, 2799000, 4999000};
unsigned long Base_SI[3] = {0xC5A58626, 0xAAB316FA, 0xB5496B5E};
unsigned long F_Step[3] = {0xD375, 0xB906, 0xB906};
unsigned long Base_LO2[2] = {0x6DBB5FE, 0x6D677D3};
unsigned long LO2_Step = 0x432;
const unsigned int Disp_Delay_Value = 8000;

//Prototypes
void InterruptHandlerHigh (void);
void Display_Freq(void);
void Parm_Set(void);
void LNAset(void);
void ADC_Parm(void);
void Display_Rest(void);
void Mode_Set(void);
void Filter_Set(void);

// Includes:
#include "p18f45k22.h"
#include "stdlib.h"
#include "delays.h"
#include "LCD Functions.c"
#include "Keypad Code.c"
#include "EEPROM_WR.c"
#include "adc.h"
#include "i2c.h"
#include "SI570.c"
#include "Config Bits.c"
#include "usart.h"
#include "DSP-610 Init.c"

// Interrupt Vector
#pragma code InterruptVectorHigh = 0x08 
void InterruptVectorHigh (void) 
{ 
  _asm 
    goto InterruptHandlerHigh //jump to interrupt routine 
  _endasm 
} 
#pragma code

// High priority interrupt routine 

#pragma interrupt InterruptHandlerHigh 
void InterruptHandlerHigh (void)
 	{
	unsigned char New_PB, Diff_PB, A, B, C;
	INTCONbits.GIE = 0;
	PORTAbits.RA4 = 1;
	if (INTCONbits.RBIF == 1)
	{
	New_PB = PORTB & 0xe0;
	Diff_PB = New_PB ^ Old_PB;
	A = New_PB & 0xc0;
	B = PORTB & 0x20;
	C = PORTB & 0x10;
	switch (Diff_PB)
		{
		case 0x80:
			if (A == 0 | A == 0xc0) Freq_Dial -= Rate;
			else Freq_Dial += Rate;
			Old_PB = New_PB;
			Freq_Store = 0x1ffff;
			Int_Flag = 1;
			break;
		case 0x40:
			if (A == 0 | A == 0xc0) Freq_Dial += Rate;
			else Freq_Dial -= Rate;
			Old_PB = New_PB;
			Freq_Store = 0x1ffff;
			Int_Flag = 1;
			break;
		case 0x20:
			if (B == 0x20)
			{
				if (C != 0) Parm_Data[Parm_Sel]++;
				else Parm_Data[Parm_Sel]--;
			}
			Old_PB = New_PB;
			Int_Flag = 2;
		}
	}
	if (INTCONbits.INT0IF == 1)
	{
		TX_Flag =1;
		if (PORTBbits.RB0 == 0) 
		{
			TX = 1;
			PORTCbits.RC0 = 1;
			INTCON2bits.INTEDG0 = 1;
			PORTAbits.RA5 = 0;
		}
		else
		{
			TX = 0;
			PORTCbits.RC0 = 0;
			CW_Delay = 10000;
			INTCON2bits.INTEDG0 = 0;
		}
	}
	PORTAbits.RA4 = 0;
	INTCON = 0x98;
 	} 
#pragma code
 

void main(void)
{
	Init_Routine();
	RCSTA1 = 0x90;

	while(1)
	{
	if (TX_Flag == 1)
	{
		TX_Flag = 0;
		LCD_PUTC (0x8b);
		if (TX == 1)
		{
			Freq_Offset = F_Step[Band] * Freq_Dial;
			Math_Temp = Freq_Offset >> 4;
			LO1.WD = Base_SI[Band] + Math_Temp;
			SlideSI570(); 
			LO2_Set();
			R_PWR = 0;
			F_PWR = 0;
			LCD_PUTD ('T');
			LCD_PUTD ('X');
			LCD_PUTD (' ');
			PORTCbits.RC2 = 1;
		}
		else
		{
			if ((Mode[Band][A_B[Band]] != 2) ||  CW_Delay == 0)
			{
				LO2_Set();
				Freq_Offset = F_Step[Band] * (Freq_Dial + RIT_Freq);
				Math_Temp = Freq_Offset >> 4;
				LO1.WD = Base_SI[Band] + Math_Temp;
				SlideSI570(); 
				if (Power == 1)	puts1USART(P1);
				else puts1USART(P0);
				SWR_E = 0;
				PORTCbits.RC2 = 0;
				if (LNA[Band] == 1)	PORTAbits.RA5 = 1;
				else PORTAbits.RA5 = 0;
				switch (Mode[Band][A_B[Band]])
				{
				case 0:
					LCD_PUTD('U');
					LCD_PUTD('S');
					LCD_PUTD('B');
					break;
				case 1:
					LCD_PUTD('L');
					LCD_PUTD('S');
					LCD_PUTD('B');
					break;
				case 2:
					LCD_PUTD('C');
					LCD_PUTD('W');
					LCD_PUTD(' ');
					break;
				}
			}
		}
	}
	if (CW_Delay > 0) 
	{
		CW_Delay--;
		if (CW_Delay == 0) TX_Flag = 1;
	}
	Key = Read_Keypad();
	TEMP = Key;
	while (TEMP != 0) TEMP = Read_Keypad();
	switch (Key)
	{
	case 1: // A<>B
		if (TX == 0)
		{
			Band_Offset[Band][A_B[Band]] = Freq_Dial;
			A_B[Band] ^= 0x01;
			WriteEEPROM(0x0e + Band,A_B[Band]);
			Freq_Dial = Band_Offset[Band][A_B[Band]];
			EEPROM_Freq (Band,0);
			EEPROM_Freq (Band,1);
			Mode_Set();
			Filter_Set();
			slide = 0;
			Display_Freq();
		}
		Delay10KTCYx(150);
		break;
	;
	case 2: // A=B
		if (TX == 0)
		{
			Not_A_B = A_B[Band] ^ 0x01;
			Band_Offset[Band][0] = Freq_Dial;
			Band_Offset[Band][1] = Freq_Dial;
			Mode[Band][Not_A_B] = Mode[Band][A_B[Band]];
			Filter[Band][Not_A_B] = Filter[Band][A_B[Band]];
			EEPROM_Freq (Band,0);
			EEPROM_Freq (Band,1);
			LCD_PUTC(0x80);
			LCD_PUTD(' ');
			LCD_PUTD(' ');
			LCD_PUTD('A');
			LCD_PUTD('=');
			LCD_PUTD('B');
			LCD_PUTD(' ');
			LCD_PUTD(' ');
			LCD_PUTD(' ');
			Disp_Delay = Disp_Delay_Value;
		}
		Delay10KTCYx(150);
		break;
	case 3: //FILTER
		if (TX == 0)
		{
			if (--Filter[Band][A_B[Band]] > 3) Filter[Band][A_B[Band]] = 3;
			WriteEEPROM(0x1b + A_B[Band] * 3 + Band, Filter[Band][A_B[Band]]);
			Filter_Set();
		}
		Delay10KTCYx(150);
		break;
	case 4: //MODE
		if (TX == 0)
		{
			if (++Mode[Band][A_B[Band]] > 2) Mode[Band][A_B[Band]] = 0;
			WriteEEPROM(0x15 + A_B[Band] * 3 + Band, Mode[Band][A_B[Band]]);
			Mode_Set();
			Filter_Set();
		}
		Delay10KTCYx(150);
		break;
	case 5: //LNA
		if (TX == 0)
		{
			LNA[Band] ^= 0x01;
			LNAset();
			WriteEEPROM(0x12 + Band,LNA[Band]);
		}
		Delay10KTCYx(150);
		break;
	case 6: //Meter
		if (TXM == 'R')
		{
			LCD_PUTC(0x80);
			LCD_PUTD('F');
			LCD_PUTD('W');
			LCD_PUTD('D');
			LCD_PUTD(' ');
			LCD_PUTD('P');
			LCD_PUTD('W');
			LCD_PUTD('R');
			LCD_PUTD(' ');
			TXM = 'F';
		}
		else
		{
			LCD_PUTC(0x80);
			LCD_PUTD('R');
			LCD_PUTD('F');
			LCD_PUTD('L');
			LCD_PUTD(' ');
			LCD_PUTD('P');
			LCD_PUTD('W');
			LCD_PUTD('R');
			LCD_PUTD(' ');
			TXM = 'R';
		}
		Delay10KTCYx(150);
		Disp_Delay = Disp_Delay_Value;
		break;
	case 7: //BAND
		if (TX == 0)
		{
			Band_Offset [Band][A_B[Band]] = Freq_Dial;
			if (++Band > 2) Band = 0;
			LNAset();
			Mode_Set();
			Filter_Set();
			WriteEEPROM(0x11,Band);
			Freq_Dial = Band_Offset [Band][A_B[Band]];
			slide = 0;
			Display_Freq();
			if (Band == 2) PORTCbits.RC1=1;
			else PORTCbits.RC1=0;
		}
		Delay10KTCYx(150);
		break;
	case 8: //LO-HI
		Power ^= 0x01;
		if (Power == 1)
		{
			LCD_PUTC(0xc6);
			LCD_PUTD('2');
			LCD_PUTD('5');
			LCD_PUTD('w');
			puts1USART(P1);
		}
		else
		{
			LCD_PUTC(0xc6);
			LCD_PUTD(' ');
			LCD_PUTD('6');
			LCD_PUTD('w');
			puts1USART(P0);
		}
		Delay10KTCYx(150);
		break;
	case 9: // DIAL
		if (++Parm_Sel > 3) Parm_Sel = 0;
		Parm_Set();
		Delay10KTCYx(150);
		if (Parm_Sel == 1)
		{
			LCD_PUTC(0x80);
			LCD_PUTD('A');
			LCD_PUTD('G');
			LCD_PUTD('C');
			LCD_PUTD(' ');
			LCD_PUTD('T');
			LCD_PUTD('I');
			LCD_PUTD('M');
			LCD_PUTD('E');
			Disp_Delay = Disp_Delay_Value;
		}	
		if (Parm_Sel == 2)
		{
			LCD_PUTC(0x80);
			LCD_PUTD('M');
			LCD_PUTD('I');
			LCD_PUTD('C');
			LCD_PUTD(' ');
			LCD_PUTD('G');
			LCD_PUTD('I');
			LCD_PUTD('A');
			LCD_PUTD('N');
			Disp_Delay = Disp_Delay_Value;
		}	
		if (Parm_Sel == 3)
		{
			LCD_PUTC(0x80);
			LCD_PUTD('S');
			LCD_PUTD('I');
			LCD_PUTD('D');
			LCD_PUTD('E');
			LCD_PUTD('T');
			LCD_PUTD('O');
			LCD_PUTD('N');
			LCD_PUTD('E');
			Disp_Delay = Disp_Delay_Value;
		}	
		Delay10KTCYx(150);
		break; 
	case 10: //RATE
		if (Rate == 100) Rate = 1;
		else Rate *= 10;
			LCD_PUTC(0x80);
			LCD_PUTD(' ');
			LCD_PUTD(' ');
		if (Rate == 1)
			{
			LCD_PUTD('S');
			LCD_PUTD('L');
			LCD_PUTD('O');
			LCD_PUTD('W ');
			}
		if (Rate == 10)
			{
			LCD_PUTD('M');
			LCD_PUTD('E');
			LCD_PUTD('D');
			LCD_PUTD(' ');
			}
		if (Rate == 100)
			{
			LCD_PUTD('F');
			LCD_PUTD('A');
			LCD_PUTD('S');
			LCD_PUTD('T');
			}
		LCD_PUTD(' ');
		LCD_PUTD(' ');
		Delay10KTCYx(150);
		Disp_Delay = Disp_Delay_Value;
		break;
	}
	if (Disp_Delay > 0)
	{
		Disp_Delay--;
		if (Disp_Delay == 1) Display_Rest();
	}
	if (Int_Flag == 1)
	{
		Int_Flag = 0;
		Display_Freq();
	}
	if (Int_Flag == 2)
	{
		Int_Flag = 0;
		Parm_Set();
	}	
	if (Freq_Store > 0)
	{
		if (Freq_Store == 1)
			{
			Band_Offset[Band][A_B[Band]] = Freq_Dial;
			EEPROM_Freq(Band,A_B[Band]);
			}
		Freq_Store--;
	}
	if (TX == 1)
	{
		SetChanADC(ADC_CH0);
		ConvertADC();
		while (BusyADC());
		M_Temp = ReadADC() << 2;
		if (M_Temp > F_PWR) F_PWR = M_Temp;
		else 
		{
			if (F_PWR < 2) F_PWR = 0;
			else F_PWR -= 2;
		}
		SetChanADC(ADC_CH1);
		ConvertADC();
		while (BusyADC());
		M_Temp = ReadADC() << 2;
		if (M_Temp > R_PWR) R_PWR = M_Temp;
		else 
		{
			if (R_PWR < 2) R_PWR = 0;
			else R_PWR -= 2;
		}
		if (R_PWR > 0x500)	
		{
			puts1USART(PE);
			SWR_E = 1;
		}
		LCD_PUTC(0xca);
		if (TXM == 'F') PWR = F_PWR >> 8;
		if (TXM == 'R') PWR = R_PWR >> 7;
		i = 0;
		while (i < 8)
		{
			if (SWR_E == 0)
			{
				if (PWR > i) LCD_PUTD(5);
				else LCD_PUTD(4);
			}
			else LCD_PUTD('-');			
			i++;
		}
		LCD_PUTD (' ');
		LCD_PUTD (TXM);
	}

	if (TX == 0);
	{
		SetChanADC(ADC_CH2);
		ConvertADC();
		while (BusyADC());
		new_RIT = ReadADC();
	
		if ((new_RIT != old1_RIT)&&(new_RIT != old2_RIT))
		{
			old2_RIT = old1_RIT;
			old1_RIT = new_RIT;
			if ((new_RIT > 499)&&(new_RIT < 524))
			{
				LCD_PUTC(0x88);
				LCD_PUTD(' ');
				RIT_Freq = 0;
				Int_Flag = 1;
			}
			if (new_RIT < 500)
			{
				LCD_PUTC(0x88);
				LCD_PUTD(7);
				RIT_Freq = new_RIT - 500;
				Int_Flag = 1;
			}
			if (new_RIT > 523)
			{
				LCD_PUTC(0x88);
				LCD_PUTD(6);
				RIT_Freq = new_RIT - 523;
				Int_Flag = 1;
			}
	 	}		
	}
	if (TX == 0);
	{
		SetChanADC(ADC_CH3);
		ConvertADC();
		while (BusyADC());
		ADC_Word = ReadADC();
		new_RFG = ADC_Word >> 3;
	
		if ((new_RFG != old1_RFG)&&(new_RFG != old2_RFG))
			{
			old2_RFG = old1_RFG;
			old1_RFG = new_RFG;
			RFG[2] = new_RFG + 0x2B;
			puts1USART (RFG);
			}
	}
	if (DataRdy1USART() && TX ==0) 
		{

		RX_char = RCREG1;
		LCD_PUTC(0xca);
		if (RX_char < 215) S_meter = 215 - RX_char;
		else S_meter = 0;

		RCSTA1 = 0x00;
		RCSTA1 = 0x90;
		i = 0;
		k = 0;
		while (i<8)
			{
			j = i * 0xa;
			if (i == 4) k = 2;
			if (S_meter >= j && S_meter < j + 0x5) LCD_PUTD(0 + k);
			if (S_meter >= j + 0x5) LCD_PUTD(1 + k);
			if (S_meter < j) LCD_PUTD(' ');
			i++;
			}
		LCD_PUTD (' ');
		LCD_PUTD ('S');
		}
	if (TX == 1)
		{

		}

	}
}

void Display_Freq(void)
{

	if (Freq_Dial < 1000) Freq_Dial = 1000;
	if (Freq_Dial > 64000) Freq_Dial = 64000;
	if (Freq_Dial < fd_lim) slide = 0;
	if (Freq_Dial > fu_lim) slide = 0;

	Freq_Offset = F_Step[Band] * (Freq_Dial + RIT_Freq);
	Math_Temp = Freq_Offset >> 4;
	LO1.WD = Base_SI[Band] + Math_Temp;

	if (slide == 0)
	{
		WriteSI570();
		if (Freq_Dial < 59000) fu_lim = Freq_Dial + 5000;
		else fu_lim = 64000;
		if (Freq_Dial > 6000) fd_lim = Freq_Dial - 5000;
		else fd_lim = 1000;
		slide = 1;
	}
	else SlideSI570(); 

	Freq_Data = Freq_Dial + Base_Freq [Band] + RIT_Freq;
	ultoa(Freq_Data, Freq_Disp);

	LCD_PUTC (0x89);
	if (A_B[Band] == 0) LCD_PUTD('A');
	else LCD_PUTD('B');
	LCD_PUTC(0x80);
	LCD_PUTD(Freq_Disp[0]);
	LCD_PUTD(Freq_Disp[1]);
	LCD_PUTD(Freq_Disp[2]);
	LCD_PUTD(Freq_Disp[3]);
	LCD_PUTD(Freq_Disp[4]);
	LCD_PUTD('.');
	LCD_PUTD(Freq_Disp[5]);
	LCD_PUTD(Freq_Disp[6]);
}
void Display_Rest(void)
{
	Freq_Data = Freq_Dial + Base_Freq [Band] + RIT_Freq;
	ultoa(Freq_Data, Freq_Disp);

	LCD_PUTC (0x89);
	if (A_B[Band] == 0) LCD_PUTD('A');
	else LCD_PUTD('B');
	LCD_PUTC(0x80);
	LCD_PUTD(Freq_Disp[0]);
	LCD_PUTD(Freq_Disp[1]);
	LCD_PUTD(Freq_Disp[2]);
	LCD_PUTD(Freq_Disp[3]);
	LCD_PUTD(Freq_Disp[4]);
	LCD_PUTD('.');
	LCD_PUTD(Freq_Disp[5]);
	LCD_PUTD(Freq_Disp[6]);
}
void Mode_Set(void)
{
unsigned char out_C[4] = "*MS";
	LCD_PUTC (0x8b);
	switch (Mode[Band][A_B[Band]])
	{
	case 0:
		LCD_PUTD('U');
		LCD_PUTD('S');
		LCD_PUTD('B');
		break;
	case 1:
		LCD_PUTD('L');
		LCD_PUTD('S');
		LCD_PUTD('B');
		break;
	case 2:
		LCD_PUTD('C');
		LCD_PUTD('W');
		LCD_PUTD(' ');
		break;
	}
	LO2_Set();
	if (Mode[Band][A_B[Band]] == 2) out_C[2] = 'C';
	else out_C[2] = 'S';
	puts1USART (out_C);
}
void Filter_Set(void)
{
unsigned char C, out_C[4] = "*B0";
	LCD_PUTC (0x8f);
	if (Mode[Band][A_B[Band]] == 2) C = Filter[Band][A_B[Band]];
	else C = Filter[Band][A_B[Band]] + 4;
	switch(C)
	{
	case 0:
		LCD_PUTD('0');
		LCD_PUTD('.');
		LCD_PUTD('4');
		break;
	case 1:
		LCD_PUTD('0');
		LCD_PUTD('.');
		LCD_PUTD('6');
		break;
	case 2:
		LCD_PUTD('0');
		LCD_PUTD('.');
		LCD_PUTD('8');
		break;
	case 3:
		LCD_PUTD('1');
		LCD_PUTD('.');
		LCD_PUTD('0');
		break;
	case 4:
		LCD_PUTD('1');
		LCD_PUTD('.');
		LCD_PUTD('8');
		break;
	case 5:
		LCD_PUTD('2');
		LCD_PUTD('.');
		LCD_PUTD('0');
		break;
	case 6:
		LCD_PUTD('2');
		LCD_PUTD('.');
		LCD_PUTD('2');
		break;
	case 7:
		LCD_PUTD('2');
		LCD_PUTD('.');
		LCD_PUTD('4');
		break;
	}
	LCD_PUTD('K');
	out_C[2] = C + 48;
	puts1USART(out_C);
}
void Parm_Set(void)
{
unsigned char out_C[4] = "*";
int P_Temp;
	LCD_PUTC (0xc0);
	switch(Parm_Sel)
	{
	case 0:
		out_C[1] = 'S';
		LCD_PUTD('P');
		LCD_PUTD('B');
		LCD_PUTD('T');
		if (Parm_Data[0] > 9) Parm_Data[0] = 9;
		if (Parm_Data[0] < -9) Parm_Data[0] = -9;
		PBT_Freq = ABS(Parm_Data[0]) * LO2_Step;
		LO2_Set();
		LCD_PUTC(0xc3);
		if (Parm_Data[0] > 0) LCD_PUTD('+');
		if (Parm_Data[0] == 0) LCD_PUTD(' ');
		if (Parm_Data[0] < 0) LCD_PUTD('-');
		if (Band == 2) P_Temp = 0 - Parm_Data[0];
		else P_Temp = Parm_Data[0];
		LCD_PUTD(ABS(Parm_Data[0]) + 48);
		out_C[2] = P_Temp + 48;
		puts1USART (out_C);
		break;
	case 1:
		out_C[1] = 'H';
		LCD_PUTD('A');
		LCD_PUTD('G');
		LCD_PUTD('C');
		LCD_PUTD(' ');
		if (Parm_Data[1] > 2) Parm_Data[1] = 2;
		if (Parm_Data[1] < 0) Parm_Data[1] = 0;
		switch (Parm_Data[1])	
		{
		case 0:
			LCD_PUTD('F');
			out_C[2] = 'S';
			break;
		case 1:
			LCD_PUTD('N');
			out_C[2] = 'M';
			break;
		case 2:
			LCD_PUTD('S');
			out_C[2] = 'L';
			break;
		}
		puts1USART (out_C);
		break;
	case 2:
		out_C[1] = 'G';
		LCD_PUTD('M');
		LCD_PUTD('I');
		LCD_PUTD('C');
		if (Parm_Data[2] > 2) Parm_Data[2] = 2;
		if (Parm_Data[2] < 0) Parm_Data[2] = 0;
		WriteEEPROM(0x22, Parm_Data[2]);
		switch (Parm_Data[2])	
		{
		case 0:
			LCD_PUTD('-');
			LCD_PUTD('6');
			out_C[2] = 'L';
			break;
		case 1:
			LCD_PUTD(' ');
			LCD_PUTD('0');
			out_C[2] = 'M';
			break;
		case 2:
			LCD_PUTD('+');
			LCD_PUTD('6');
			out_C[2] = 'H';
			break;
		}
		puts1USART (out_C);
		break;
	case 3:
		out_C[1] = 'T';
		LCD_PUTD('S');
		LCD_PUTD('T');
		LCD_PUTD('L');
		LCD_PUTD(' ');
		if (Parm_Data[3] > 9) Parm_Data[3] = 9;
		if (Parm_Data[3] < 1) Parm_Data[3] = 1;
		WriteEEPROM(0x21, Parm_Data[3]);
		out_C[2] = Parm_Data[3] + 48;
		LCD_PUTD(Parm_Data[3] + 48);
		puts1USART (out_C);
		break;
	}
}
void ADC_Parm(void)
{
	unsigned char c=0;
	LCD_PUTC(0xc7);
	itoa(Filter[Band][A_B[Band]], Parm_Disp);

	while (Parm_Disp[c] != 0)
	{
		LCD_PUTD(Parm_Disp[c++]);
	}
	LCD_PUTD(' ');
}

void LNAset(void)
{
	if (LNA[Band] == 1) 
	{
	PORTAbits.RA5 = 1;
	LCD_PUTC(0x80);
	LCD_PUTD('L');
	LCD_PUTD('N');
	LCD_PUTD('A');
	LCD_PUTD(' ');
	LCD_PUTD('O');
	LCD_PUTD('N');
	LCD_PUTD(' ');
	LCD_PUTD(' ');
	LCD_PUTC(0x93);
	LCD_PUTD('p');
	}
	else 
	{
	PORTAbits.RA5 = 0;
	LCD_PUTC(0x80);
	LCD_PUTD('L');
	LCD_PUTD('N');
	LCD_PUTD('A');
	LCD_PUTD(' ');
	LCD_PUTD('O');
	LCD_PUTD('F');
	LCD_PUTD('F');
	LCD_PUTD(' ');
	LCD_PUTC(0x93);
	LCD_PUTD(' ');
	}

	Disp_Delay = Disp_Delay_Value;
}
